@@ -16,7 +16,8 @@ gi.require_version("Gst", "1.0")
 gi.require_version("GstAudio", "1.0")
 gi.require_version("GstPbutils", "1.0")
 gi.require_version("TotemPlParser", "1.0")
-from gi.repository import Gtk, Gio, GLib, Gdk, TotemPlParser
+from gi.repository import Gtk, Gio, GLib, Gdk, Gst
+Gst.init(None)
 
 from threading import current_thread
 from pickle import dump
@@ -54,6 +55,7 @@ from lollypop.notification import NotificationManager
 from lollypop.playlists import Playlists
 from lollypop.objects import Album, Track
 from lollypop.helper_task import TaskHelper
+from lollypop.helper_art import ArtHelper
 from lollypop.collectionscanner import CollectionScanner
 
 
@@ -108,6 +110,7 @@ class Application(Gtk.Application):
         self.notify = None
         self.scrobblers = []
         self.debug = False
+        self.shown_sidebar_tooltip = False
         self.__fs = None
         GLib.set_application_name("Lollypop")
         GLib.set_prgname("lollypop")
@@ -186,6 +189,7 @@ class Application(Gtk.Application):
         self.notify = NotificationManager()
         self.art.update_art_size()
         self.task_helper = TaskHelper()
+        self.art_helper = ArtHelper()
         if self.settings.get_value("artist-artwork"):
             GLib.timeout_add(5000, self.art.cache_artists_info)
         # Load lastfm if support available
@@ -204,6 +208,12 @@ class Application(Gtk.Application):
             dark = self.settings.get_value("dark-ui")
             settings.set_property("gtk-application-prefer-dark-theme", dark)
         ApplicationActions()
+        startup_one_ids = self.settings.get_value("startup-one-ids")
+        startup_two_ids = self.settings.get_value("startup-two-ids")
+        if startup_one_ids:
+            self.settings.set_value("state-one-ids", startup_one_ids)
+        if startup_two_ids:
+            self.settings.set_value("state-two-ids", startup_two_ids)
 
     def do_startup(self):
         """
@@ -217,19 +227,16 @@ class Application(Gtk.Application):
             self.window.connect("delete-event", self.__hide_on_delete)
             self.window.show()
             self.player.restore_state()
-            # We add to mainloop as we want to run
-            # after player::restore_state() signals
-            GLib.idle_add(self.window.toolbar.set_mark)
-            self.__preload_portal()
 
     def quit(self, vacuum=False):
         """
             Quit Lollypop
             @param vacuum as bool
         """
+        if not self.settings.get_value("save-state"):
+            self.settings.set_value("state-one-ids", GLib.Variant("ai", []))
+            self.settings.set_value("state-two-ids", GLib.Variant("ai", []))
         self.window.container.save_internals()
-        if self.settings.get_value("save-state"):
-            self.window.container.save_view_state()
         # Then vacuum db
         if vacuum:
             self.__vacuum()
@@ -319,10 +326,10 @@ class Application(Gtk.Application):
             # Save current playlist
             if self.player.current_track.id == Type.RADIOS:
                 playlist_ids = [Type.RADIOS]
-            elif not self.player.get_playlist_ids():
+            elif not self.player.playlist_ids:
                 playlist_ids = []
             else:
-                playlist_ids = self.player.get_playlist_ids()
+                playlist_ids = self.player.playlist_ids
             dump(playlist_ids,
                  open(LOLLYPOP_DATA_PATH + "/playlist_ids.bin", "wb"))
         if self.player.current_track.id is not None:
@@ -359,21 +366,6 @@ class Application(Gtk.Application):
         except Exception as e:
             Logger.error("Application::__vacuum(): %s" % e)
 
-    def __preload_portal(self):
-        """
-            Preload lollypop portal
-        """
-        try:
-            bus = self.get_dbus_connection()
-            Gio.DBusProxy.new(bus, Gio.DBusProxyFlags.NONE, None,
-                              "org.gnome.Lollypop.Portal",
-                              "/org/gnome/LollypopPortal",
-                              "org.gnome.Lollypop.Portal", None, None)
-        except Exception as e:
-            Logger.info("You are missing lollypop-portal: "
-                        "https://github.com/gnumdk/lollypop-portal")
-            Logger.error("Application::__preload_portal(): %s", e)
-
     def __on_handle_local_options(self, app, options):
         """
             Handle local options
@@ -445,6 +437,7 @@ class Application(Gtk.Application):
                 elif is_pls(f):
                     pls.append(uri)
             if pls:
+                from gi.repository import TotemPlParser
                 parser = TotemPlParser.Parser.new()
                 parser.connect("entry-parsed", self.__on_entry_parsed, uris)
                 parser.parse_async(uri, True, None,
@@ -452,7 +445,7 @@ class Application(Gtk.Application):
             else:
                 self.__on_parse_finished(None, None, uris)
         elif self.window is not None:
-            self.window.setup_window()
+            self.window.setup()
             if not self.window.is_visible():
                 # https://bugzilla.gnome.org/show_bug.cgi?id=766284
                 monotonic_time = int(GLib.get_monotonic_time() / 1000)
@@ -469,7 +462,7 @@ class Application(Gtk.Application):
             @param result as Gio.AsyncResult
             @param uris as [str]
         """
-        self.scanner.update(uris, False)
+        GLib.timeout_add(500, self.scanner.update, uris, False)
 
     def __on_entry_parsed(self, parser, uri, metadata, uris):
         """
