@@ -1,10 +1,4 @@
 # Copyright (c) 2014-2018 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
-# Copyright (c) 2013 Vadim Rutkovsky <vrutkovs@redhat.com>
-# Copyright (c) 2013 Arnel A. Borja <kyoushuu@yahoo.com>
-# Copyright (c) 2013 Seif Lotfy <seif@lotfy.com>
-# Copyright (c) 2013 Guillaume Quintard <guillaume.quintard@gmail.com>
-# Copyright (c) 2013 Lubosz Sarnecki <lubosz@gmail.com>
-# Copyright (c) 2013 Sai Suman Prayaga <suman.sai14@gmail.com>
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
@@ -16,7 +10,7 @@
 # You should have received a copy of the GNU General Public License
 # along with this program. If not, see <http://www.gnu.org/licenses/>.
 
-from gi.repository import Gtk, Gdk, GObject, GdkPixbuf, Gio, GLib
+from gi.repository import GObject, GdkPixbuf, Gio, GLib
 
 from lollypop.define import ArtSize, App, TAG_EDITORS
 from lollypop.logger import Logger
@@ -70,63 +64,6 @@ class BaseArt(GObject.GObject):
         except Exception as e:
             Logger.error("Art::clean_store(): %s" % e)
 
-    def get_default_icon(self, icon_name, size, scale):
-        """
-            Construct an empty cover album,
-            code forked Gnome Music, see copyright header
-            @param icon_name as str
-            @param size as int
-            @param scale factor as int
-            @return pixbuf as cairo.Surface
-        """
-        try:
-            # First look in cache
-            cache_path_jpg = self._get_default_icon_path(size, icon_name)
-            f = Gio.File.new_for_path(cache_path_jpg)
-            if f.query_exists():
-                pixbuf = GdkPixbuf.Pixbuf.new_from_file_at_scale(
-                    cache_path_jpg,
-                    size,
-                    size,
-                    False)
-            else:
-                # get a small pixbuf with the given path
-                icon_size = size / 4
-                icon = Gtk.IconTheme.get_default().load_icon(icon_name,
-                                                             icon_size, 0)
-                # create an empty pixbuf with the requested size
-                pixbuf = GdkPixbuf.Pixbuf.new(icon.get_colorspace(),
-                                              True,
-                                              icon.get_bits_per_sample(),
-                                              size,
-                                              size)
-                pixbuf.fill(0xffffffff)
-                icon.composite(pixbuf,
-                               icon_size * 3 / 2,
-                               icon_size * 3 / 2,
-                               icon_size,
-                               icon_size,
-                               icon_size * 3 / 2,
-                               icon_size * 3 / 2,
-                               1, 1,
-                               GdkPixbuf.InterpType.NEAREST, 255)
-                # Gdk < 3.15 was missing save method
-                # > 3.15 is missing savev method
-                try:
-                    pixbuf.save(cache_path_jpg, "jpeg",
-                                ["quality"], [str(App().settings.get_value(
-                                              "cover-quality").get_int32())])
-                except:
-                    pixbuf.savev(cache_path_jpg, "jpeg",
-                                 ["quality"], [str(App().settings.get_value(
-                                     "cover-quality").get_int32())])
-            surface = Gdk.cairo_surface_create_from_pixbuf(pixbuf, scale, None)
-            return surface
-        except:
-            return self.get_default_icon("computer-fail-symbolic",
-                                         ArtSize.MEDIUM,
-                                         scale)
-
     @property
     def kid3_available(self):
         """
@@ -146,32 +83,30 @@ class BaseArt(GObject.GObject):
 #######################
 # PROTECTED           #
 #######################
-    def _preserve_ratio(self, uri):
+    def _preserve_ratio(self, pixbuf, w, h):
         """
-            Preserve the cover’s aspect ratio
+            Return scaled pixbuf if needed
             @param uri as str
-            @return respect aspect ratio as bool
+            @param w as int
+            @param h as int
+            @return GdkPixbuf.Pixbuf
         """
         if App().settings.get_value("preserve-aspect-ratio"):
-            return True
-        f = Gio.File.new_for_uri(uri)
-        (status, data, tag) = f.load_contents(None)
-        bytes = GLib.Bytes(data)
-        stream = Gio.MemoryInputStream.new_from_bytes(bytes)
-        bytes.unref()
-        cover = GdkPixbuf.Pixbuf.new_from_stream(stream, None)
-        stream.close()
-        cover_width = cover.get_width()
-        cover_height = cover.get_height()
-        del cover
-        if cover_width == cover_height:
-            return True
-        elif cover_width < cover_height:
-            cut = cover_height / 5
-            return cover_width < cover_height - cut
+            return pixbuf
+        width = pixbuf.get_width()
+        height = pixbuf.get_height()
+        if width == height:
+            preserve = True
+        elif width < height:
+            cut = height / 5
+            preserve = width < height - cut
         else:
-            cut = cover_width / 5
-            return cover_height < cover_width - cut
+            cut = width / 5
+            preserve = height < width - cut
+        if preserve:
+            return pixbuf
+        else:
+            return pixbuf.scale_simple(w, h, GdkPixbuf.InterpType.BILINEAR)
 
     def _create_store(self):
         """
@@ -195,13 +130,6 @@ class BaseArt(GObject.GObject):
             except:
                 Logger.info("Can't create %s" % self._CACHE_PATH)
 
-    def _get_default_icon_path(self, size, icon_name):
-        """
-            Return default icon path
-            @return path as string
-        """
-        return "%s/%s_%s.jpg" % (self._CACHE_PATH, icon_name, size)
-
 #######################
 # PRIVATE             #
 #######################
