@@ -28,9 +28,13 @@ class CellRendererAlbum(Gtk.CellRenderer):
     def do_render(self, ctx, widget, background_area, cell_area, flags):
         if self.album == Type.NONE:
             return
-        surface = App().art.get_album_artwork(Album(self.album),
-                                              ArtSize.MEDIUM,
-                                              widget.get_scale_factor())
+        scale_factor = widget.get_scale_factor()
+        pixbuf = App().art.get_album_artwork(Album(self.album),
+                                             ArtSize.MEDIUM,
+                                             ArtSize.MEDIUM,
+                                             scale_factor)
+        surface = Gdk.cairo_surface_create_from_pixbuf(
+                    pixbuf, scale_factor, None)
         width = surface.get_width()
         height = surface.get_height()
         # If cover smaller than wanted size, translate
@@ -112,7 +116,8 @@ class CellRendererArtist(Gtk.CellRendererText):
         if self.rowid in self.__surfaces.keys():
             surface = self.__surfaces[self.rowid]
         if surface is None:
-            path = InformationStore.get_artwork_path(self.artist, size)
+            path = InformationStore.get_artwork_path(
+                self.artist, size, self.__scale_factor)
             if path is not None:
                 pixbuf = GdkPixbuf.Pixbuf.new_from_file_at_size(path,
                                                                 size,
