@@ -32,7 +32,7 @@ class TracksDatabase:
         pass
 
     def add(self, name, uri, duration, tracknumber, discnumber, discname,
-            album_id, year, timestamp, popularity, rate, ltime, mtime,
+            album_id, year, timestamp, popularity, rate, loved, ltime, mtime,
             mb_track_id):
         """
             Add a new track to database
@@ -48,6 +48,7 @@ class TracksDatabase:
             @param timestamp as int
             @param popularity as int
             @param rate as int
+            @param loved as bool
             @param ltime as int
             @param mtime as int
             @param mb_track_id as str
@@ -58,9 +59,9 @@ class TracksDatabase:
             result = sql.execute(
                 "INSERT INTO tracks (name, uri, duration, tracknumber,\
                 discnumber, discname, album_id,\
-                year, timestamp, popularity, rate,\
+                year, timestamp, popularity, rate, loved,\
                 ltime, mtime, mb_track_id) VALUES\
-                (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)", (
+                (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)", (
                     name,
                     uri,
                     duration,
@@ -72,6 +73,7 @@ class TracksDatabase:
                     timestamp,
                     popularity,
                     rate,
+                    loved,
                     ltime,
                     mtime,
                     mb_track_id))
@@ -144,7 +146,7 @@ class TracksDatabase:
 
     def get_id_by(self, name, album_id, artist_ids):
         """
-            Return track id for uri
+            Return track id for name/album/artists
             @param name as str
             @param album id as int
             @return track id as int
@@ -168,6 +170,20 @@ class TracksDatabase:
                 return v[0]
             return None
 
+    def get_ids_by_artist(self, artist_id):
+        """
+            Return track id for artist_id
+            @param artist_id as int
+            @return [int]
+        """
+        with SqlCursor(App().db) as sql:
+            filters = (artist_id,)
+            request = "SELECT tracks.rowid FROM tracks, track_artists\
+                       WHERE track_artists.artist_id=? AND\
+                       tracks.rowid = track_artists.track_id"
+            result = sql.execute(request, filters)
+            return list(itertools.chain(*result))
+
     def get_name(self, track_id):
         """
             Get track name for track id
@@ -621,11 +637,8 @@ class TracksDatabase:
             @param popularity as int
         """
         with SqlCursor(App().db, True) as sql:
-            try:
-                sql.execute("UPDATE tracks set popularity=? WHERE rowid=?",
-                            (popularity, track_id))
-            except:  # Database is locked
-                pass
+            sql.execute("UPDATE tracks set popularity=? WHERE rowid=?",
+                        (popularity, track_id))
 
     def get_popularity(self, track_id):
         """
@@ -641,6 +654,30 @@ class TracksDatabase:
                 return v[0]
             return 0
 
+    def get_loved(self, track_id):
+        """
+            Get track loved status
+            @param track_id as int
+            @return loved as int
+        """
+        with SqlCursor(App().db) as sql:
+            result = sql.execute("SELECT loved FROM tracks WHERE\
+                                 rowid=?", (track_id,))
+
+            v = result.fetchone()
+            if v is not None:
+                return v[0]
+            return 0
+
+    def get_loved_track_ids(self):
+        """
+            Get loved track ids
+            @return [int]
+        """
+        with SqlCursor(App().db) as sql:
+            result = sql.execute("SELECT rowid FROM tracks WHERE loved=1")
+            return list(itertools.chain(*result))
+
     def get_ltime(self, track_id):
         """
             Get listen time
@@ -670,6 +707,17 @@ class TracksDatabase:
                 return v[0]
             return 0
 
+    def set_loved(self, track_id, loved):
+        """
+            Set track loved
+            @param track_id as int
+            @param loved as int
+            @warning: commit needed
+        """
+        with SqlCursor(App().db, True) as sql:
+            sql.execute("UPDATE tracks SET loved=? WHERE rowid=?",
+                        (loved, track_id))
+
     def count(self):
         """
             Count tracks
