@@ -10,11 +10,10 @@
 # You should have received a copy of the GNU General Public License
 # along with this program. If not, see <http://www.gnu.org/licenses/>.
 
-from gi.repository import Gst
+from gi.repository import Gst, GLib
 
 from pickle import load
-from random import choice
-
+from random import choice, shuffle
 from lollypop.player_bin import BinPlayer
 from lollypop.player_queue import QueuePlayer
 from lollypop.player_linear import LinearPlayer
@@ -54,9 +53,8 @@ class Player(BinPlayer, QueuePlayer, PlaylistPlayer, RadioPlayer,
         """
         if self._is_locked:
             return
-        smart_prev = App().settings.get_value("smart-previous")
         if self._prev_track.id is not None:
-            if smart_prev and self.position / Gst.SECOND > 2:
+            if self.position / Gst.SECOND > 2:
                 self.seek(0)
             else:
                 self.load(self._prev_track)
@@ -95,7 +93,7 @@ class Player(BinPlayer, QueuePlayer, PlaylistPlayer, RadioPlayer,
 
     def add_album(self, album, index=-1):
         """
-            Add album
+            Add album to player. We may merge album!
             @param album as Album
             @param index as int
         """
@@ -105,7 +103,11 @@ class Player(BinPlayer, QueuePlayer, PlaylistPlayer, RadioPlayer,
         # We do not shuffle when user add an album
         self._albums_backup = []
         if index == -1:
-            self._albums.append(album)
+            if self._albums and self._albums[-1].id == album.id:
+                self._albums[-1].set_tracks(self._albums[-1].tracks +
+                                            album.tracks)
+            else:
+                self._albums.append(album)
         else:
             self._albums.insert(index, album)
         if self._current_track.id is not None and self._current_track.id > 0:
@@ -113,15 +115,7 @@ class Player(BinPlayer, QueuePlayer, PlaylistPlayer, RadioPlayer,
                 self.set_next()
             self.set_prev()
         self.emit("album-added", album.id)
-
-    def move_album(self, album, position):
-        """
-            Move album to position
-            @param album as Album
-            @param position as int
-        """
-        index = self._albums.index(album)
-        self._albums.insert(position, self._albums.pop(index))
+        self.emit("playlist-changed")
 
     def remove_album(self, album):
         """
@@ -136,34 +130,51 @@ class Player(BinPlayer, QueuePlayer, PlaylistPlayer, RadioPlayer,
                 self.set_next()
             self.set_prev()
             self.emit("album-removed", album.id)
+            self.emit("playlist-changed")
         except Exception as e:
             Logger.error("Player::remove_album(): %s" % e)
 
     def remove_album_by_id(self, album_id):
         """
-            Remove album id from albums
+            Remove all instance of album with id from albums
             @param album_id as int
         """
         try:
-            idx = self.album_ids.index(album_id)
-            album = self._albums[idx]
-            self._albums.remove(album)
-            if album in self._albums_backup:
-                self._albums_backup.remove(album)
-            if not self.is_party or self._next_track.album_id == album.id:
-                self.set_next()
-            self.set_prev()
-            self.emit("album-removed", album.id)
+            for album in self._albums:
+                if album.id == album_id:
+                    self.remove_album(album)
+            self.emit("playlist-changed")
         except Exception as e:
             Logger.error("Player::remove_album_by_id(): %s" % e)
 
+    def remove_disc(self, disc, album_id):
+        """
+            Remove disc for album_id
+            @param disc as Disc
+            @param album_id as int
+        """
+        try:
+            removed = []
+            for album in self._albums:
+                if album.id == album_id:
+                    for track in list(album.tracks):
+                        if track.id in disc.track_ids:
+                            empty = album.remove_track(track)
+                            if empty:
+                                removed.append(album)
+            for album in removed:
+                self._albums.remove(album)
+            self.emit("playlist-changed")
+        except Exception as e:
+            Logger.error("Player::remove_disc(): %s" % e)
+
     def play_album(self, album):
         """
             Play album
             @param album as Album
         """
         if self.is_party:
-            self.set_party(False)
+            App().lookup_action("party").change_state(GLib.Variant("b", False))
         self.reset_history()
         # We are not playing a user playlist anymore
         self._playlist_tracks = []
@@ -174,16 +185,17 @@ class Player(BinPlayer, QueuePlayer, PlaylistPlayer, RadioPlayer,
             track = album.tracks[0]
         self.load(track)
         self._albums = [album]
+        self.emit("playlist-changed")
 
-    def play_albums(self, track, filter1_ids, filter2_ids):
+    def play_albums(self, album_id, filter1_ids, filter2_ids):
         """
             Play albums related to track/genre_ids/artist_ids
-            @param track as Track/None
+            @param album_id as int/None
             @param filter1_ids as [int]
             @param filter2_ids as [int]
         """
         self._albums = []
-        album_ids = []
+        album_ids = [] if album_id is None else [album_id]
         self.reset_history()
         # We are not playing a user playlist anymore
         self._playlist_tracks = []
@@ -193,33 +205,33 @@ class Player(BinPlayer, QueuePlayer, PlaylistPlayer, RadioPlayer,
            (filter2_ids and filter2_ids[0] == Type.ALL):
             # Genres: all, Artists: compilations
             if filter2_ids and filter2_ids[0] == Type.COMPILATIONS:
-                album_ids = App().albums.get_compilation_ids()
+                album_ids += App().albums.get_compilation_ids([], True)
             # Genres: all, Artists: ids
             elif filter2_ids and filter2_ids[0] != Type.ALL:
-                album_ids += App().albums.get_ids(filter2_ids)
+                album_ids += App().albums.get_ids(filter2_ids, [], True)
             # Genres: all, Artists: all
             else:
                 if App().settings.get_value("show-compilations-in-album-view"):
-                    album_ids += App().albums.get_compilation_ids()
-                album_ids += App().albums.get_ids()
+                    album_ids += App().albums.get_compilation_ids([], True)
+                album_ids += App().albums.get_ids([], [], True)
         # We are in populars view, add popular albums
         elif filter1_ids and filter1_ids[0] == Type.POPULARS:
-            album_ids = App().albums.get_populars()
+            album_ids += App().albums.get_populars()
         # We are in loved view, add loved albums
         elif filter1_ids and filter1_ids[0] == Type.LOVED:
-            album_ids = App().albums.get_loves()
+            album_ids += App().albums.get_loved_albums()
         # We are in recents view, add recent albums
         elif filter1_ids and filter1_ids[0] == Type.RECENTS:
-            album_ids = App().albums.get_recents()
+            album_ids += App().albums.get_recents()
         # We are in randoms view, add random albums
         elif filter1_ids and filter1_ids[0] == Type.RANDOMS:
-            album_ids = App().albums.get_cached_randoms()
+            album_ids += App().albums.get_cached_randoms()
         # We are in compilation view without genre
         elif filter1_ids and filter1_ids[0] == Type.COMPILATIONS:
-            album_ids = App().albums.get_compilation_ids()
+            album_ids += App().albums.get_compilation_ids([])
         # We are in years view
         elif filter1_ids and filter1_ids[0] == Type.YEARS:
-            album_ids = []
+            album_ids += []
             for year in filter2_ids:
                 album_ids += App().albums.get_albums_for_year(year)
                 album_ids += App().albums.get_compilations_for_year(year)
@@ -228,31 +240,64 @@ class Player(BinPlayer, QueuePlayer, PlaylistPlayer, RadioPlayer,
             # If we are not in compilation view and show compilation is on,
             # add compilations
             if filter2_ids and filter2_ids[0] == Type.COMPILATIONS:
-                album_ids += App().albums.get_compilation_ids(filter1_ids)
-            else:
-                if not filter2_ids and\
-                        App().settings.get_value(
+                album_ids += App().albums.get_compilation_ids(
+                    filter1_ids, True)
+            elif filter2_ids:
+                # In artist view, play all albums if ignoring return []
+                album_ids += App().albums.get_ids(
+                    filter2_ids, filter1_ids, True)
+                if not album_ids:
+                    album_ids += App().albums.get_ids(
+                        filter2_ids, filter1_ids, False)
+            elif App().settings.get_value(
                             "show-compilations-in-album-view"):
-                    album_ids += App().albums.get_compilation_ids(filter1_ids)
-                album_ids += App().albums.get_ids(filter2_ids, filter1_ids)
+                album_ids += App().albums.get_compilation_ids(
+                    filter1_ids, True)
+                album_ids += App().albums.get_ids([], filter1_ids, True)
+            else:
+                album_ids += App().albums.get_ids([], filter1_ids, True)
+
+        shuffle_setting = App().settings.get_enum("shuffle")
+        if not album_ids:
+            return
+        elif shuffle_setting == Shuffle.ALBUMS:
+            if album_id is None:
+                shuffle(album_ids)
+                album = Album(album_ids.pop(0), filter1_ids, filter2_ids, True)
+            else:
+                album = Album(album_ids.pop(0), filter1_ids, filter2_ids, True)
+                shuffle(album_ids)
+        else:
+            album = Album(album_ids.pop(0), filter1_ids, filter2_ids, True)
+        # Select a track and start playback
+        track = None
+        self._albums = [album]
+        if shuffle_setting == Shuffle.TRACKS:
+            track = choice(album.tracks)
+        elif shuffle_setting == Shuffle.ALBUMS:
+            if self._albums and self._albums[0].tracks:
+                track = self._albums[0].tracks[0]
+        elif album.tracks:
+            track = album.tracks[0]
+
         # Create album objects
-        for album_id in album_ids:
-            album = Album(album_id, filter1_ids, filter2_ids)
+        for _album_id in album_ids:
+            if _album_id == album_id:
+                continue
+            album = Album(_album_id, filter1_ids, filter2_ids, True)
             self._albums.append(album)
-            # Get track from album
-            # to make Player.current_track present in Player.albums
-            if track is None:
-                track = album.tracks[0]
-            elif album.id == track.album.id:
-                index = album.track_ids.index(track.id)
-                track = album.tracks[index]
-        self.load(track)
+        if track is not None:
+            self.load(track)
+        self.emit("playlist-changed")
 
     def clear_albums(self):
         """
             Clear all albums
         """
         self._albums = []
+        self.set_next()
+        self.set_prev()
+        self.emit("playlist-changed")
 
     def get_current_artists(self):
         """
@@ -272,10 +317,11 @@ class Player(BinPlayer, QueuePlayer, PlaylistPlayer, RadioPlayer,
         """
         try:
             if App().settings.get_value("save-state"):
-                track_id = load(open(LOLLYPOP_DATA_PATH +
-                                     "/track_id.bin", "rb"))
+                current_track_id = load(open(LOLLYPOP_DATA_PATH +
+                                             "/track_id.bin", "rb"))
                 self.set_queue(load(open(LOLLYPOP_DATA_PATH +
                                          "/queue.bin", "rb")))
+                albums = load(open(LOLLYPOP_DATA_PATH + "/Albums.bin", "rb"))
                 playlist_ids = load(open(LOLLYPOP_DATA_PATH +
                                          "/playlist_ids.bin", "rb"))
                 (is_playing, was_party) = load(open(LOLLYPOP_DATA_PATH +
@@ -283,41 +329,46 @@ class Player(BinPlayer, QueuePlayer, PlaylistPlayer, RadioPlayer,
                 if playlist_ids and playlist_ids[0] == Type.RADIOS:
                     radios = Radios()
                     track = Track()
-                    name = radios.get_name(track_id)
+                    name = radios.get_name(current_track_id)
                     url = radios.get_url(name)
                     track.set_radio(name, url)
                     self.load(track, is_playing)
-                elif App().tracks.get_uri(track_id) != "":
-                    for playlist_id in playlist_ids:
-                        track_ids = App().playlists.get_track_ids(
-                            playlist_id)
-                        tracks = [Track(track_id)
-                                  for track_id in track_ids]
-                        wanted = None
-                        for track in tracks:
-                            if track.id == track_id:
-                                wanted = track
-                                break
-                        if wanted is not None:
-                            self._load_track(wanted)
-                        App().player.populate_playlist_by_tracks(tracks,
-                                                                 playlist_ids)
-                    else:
+                elif App().tracks.get_uri(current_track_id) != "":
+                    if albums:
                         if was_party:
-                            self.emit("party-changed", True)
+                            App().lookup_action("party").change_state(
+                                GLib.Variant("b", True))
                         else:
                             self._albums = load(open(
                                                 LOLLYPOP_DATA_PATH +
                                                 "/Albums.bin",
                                                 "rb"))
-                    self.set_next()
-                    self.set_prev()
+                        # Load track from player albums
+                        current_track = Track(current_track_id)
+                        index = self.album_ids.index(current_track.album.id)
+                        for track in self._albums[index].tracks:
+                            if track.id == current_track_id:
+                                self._load_track(track)
+                                break
+                    else:
+                        for playlist_id in playlist_ids:
+                            tracks = App().playlists.get_tracks(playlist_id)
+                            App().player.populate_playlist_by_tracks(
+                                tracks, playlist_ids)
+                            for track in tracks:
+                                if track.id == current_track_id:
+                                    self._load_track(track)
+                                    break
                     if is_playing:
                         self.play()
                     else:
                         self.pause()
+                    position = load(open(LOLLYPOP_DATA_PATH + "/position.bin",
+                                    "rb"))
+                    self.seek(position / Gst.SECOND)
                 else:
                     Logger.info("Player::restore_state(): track missing")
+                self.emit("playlist-changed")
         except Exception as e:
             Logger.error("Player::restore_state(): %s" % e)
 
@@ -427,21 +478,13 @@ class Player(BinPlayer, QueuePlayer, PlaylistPlayer, RadioPlayer,
                         return True
         return False
 
-    def object_by_name(self, track_name, album_name):
+    def get_albums_for_id(self, album_id):
         """
-            Get track by object name
-            @track_name as str
-            @param album_name as str
-            @return Album is track_name is None, else Track or None
+            Get albums for id
+            @param album_id as int
+            @return [Album]
         """
-        for album in self._albums:
-            if str(album) == album_name:
-                if track_name is None:
-                    return album
-                for track in album.tracks:
-                    if str(track) == track_name:
-                        return track
-        return None
+        return [album for album in self._albums if album.id == album_id]
 
     @property
     def next_track(self):
@@ -485,7 +528,7 @@ class Player(BinPlayer, QueuePlayer, PlaylistPlayer, RadioPlayer,
         if self._current_track.id is not None and self._current_track.id >= 0:
             ShufflePlayer._on_stream_start(self, bus, message)
         if self.track_in_queue(self._current_track):
-            self.del_from_queue(self._current_track.id)
+            self.remove_from_queue(self._current_track.id)
         else:
             if self.shuffle_has_next or not self.__do_not_update_next:
                 self.set_next()
