@@ -69,7 +69,7 @@ class RoundedArtistsView(FlowBoxView):
 
     @property
     def should_destroy(self):
-        return App().settings.get_value("show-sidebar")
+        return False
 
 #######################
 # PROTECTED           #
