@@ -10,12 +10,13 @@
 # You should have received a copy of the GNU General Public License
 # along with this program. If not, see <http://www.gnu.org/licenses/>.
 
-from gi.repository import Gtk
+from gi.repository import Gtk, GLib
 
 from gettext import gettext as _
+from locale import strcoll
 
 from lollypop.view_flowbox import FlowBoxView
-from lollypop.define import App
+from lollypop.define import App, Type
 from lollypop.widgets_playlist_rounded import PlaylistRoundedWidget
 from lollypop.shown import ShownPlaylists
 
@@ -31,29 +32,16 @@ class PlaylistsManagerView(FlowBoxView):
             @param obj as Track/Album
         """
         self.__obj = obj
-        grid = Gtk.Grid()
-        grid.set_margin_top(5)
-        grid.set_margin_start(5)
-        grid.set_margin_bottom(5)
-        grid.show()
-        if obj is not None:
-            back_button = Gtk.Button.new_from_icon_name("go-previous-symbolic",
-                                                        Gtk.IconSize.BUTTON)
-            back_button.connect("clicked",
-                                lambda x: App().window.container.reload_view())
-            back_button.set_property("halign", Gtk.Align.START)
-            back_button.set_relief(Gtk.ReliefStyle.NONE)
-            back_button.show()
-            grid.add(back_button)
+        self.__signal_id = None
         new_playlist_button = Gtk.Button(_("New playlist"))
         new_playlist_button.connect("clicked", self.__on_new_button_clicked)
         new_playlist_button.set_property("halign", Gtk.Align.CENTER)
         new_playlist_button.set_hexpand(True)
+        new_playlist_button.set_margin_top(5)
         new_playlist_button.show()
-        grid.add(new_playlist_button)
         FlowBoxView.__init__(self)
         self.insert_row(0)
-        self.attach(grid, 0, 0, 1, 1)
+        self.attach(new_playlist_button, 0, 0, 1, 1)
         self._widget_class = PlaylistRoundedWidget
 
     def populate(self, items):
@@ -61,20 +49,16 @@ class PlaylistsManagerView(FlowBoxView):
             Populate items
             @param items
         """
-        if self.__obj is None:
+        if self.__obj is not None:
+            new_items = []
+            for item in items:
+                if not App().playlists.get_smart(item):
+                    new_items.append(item)
+            items = new_items
+        else:
             items = [i[0] for i in ShownPlaylists.get()] + items
         FlowBoxView.populate(self, items)
 
-    def get_track_ids_for_playlist_id(self, playlist_id):
-        """
-            Get track_ids for current playlist id (even random)
-            @return [int]
-        """
-        for child in self._box.get_children():
-            if child.playlist_id == playlist_id:
-                return child.track_ids
-        return []
-
 #######################
 # PROTECTED           #
 #######################
@@ -88,16 +72,84 @@ class PlaylistsManagerView(FlowBoxView):
         if widget is not None:
             widget.connect("overlayed", self.on_overlayed)
 
+    def _on_map(self, widget):
+        """
+            Setup widget
+        """
+        self.__signal_id = App().playlists.connect("playlists-changed",
+                                                   self.__on_playlist_changed)
+        if self.__obj is None:
+            App().settings.set_value("state-one-ids",
+                                     GLib.Variant("ai", [Type.PLAYLISTS]))
+            App().settings.set_value("state-two-ids",
+                                     GLib.Variant("ai", []))
+        else:
+            App().window.emit("can-go-back-changed", True)
+            App().window.emit("show-can-go-back", True)
+
+    def _on_unmap(self, widget):
+        """
+            Disconnect signal
+            @param widget as Gtk.Widget
+        """
+        if self.__signal_id is not None:
+            App().playlists.disconnect(self.__signal_id)
+            self.__signal_id = None
+
 #######################
 # PRIVATE             #
 #######################
+    def __sort_func(self, widget1, widget2):
+        """
+            Sort function
+            @param widget1 as PlaylistRoundedWidget
+            @param widget2 as PlaylistRoundedWidget
+        """
+        # Static vs static
+        if widget1.data < 0 and widget2.data < 0:
+            return widget1.data < widget2.data
+        # Static entries always on top
+        elif widget2.data < 0:
+            return True
+        # Static entries always on top
+        if widget1.data < 0:
+            return False
+        # String comparaison for non static
+        else:
+            return strcoll(widget1.name, widget2.name)
+
+    def __on_playlist_changed(self, playlists, playlist_id):
+        """
+            Update view based on playlist_id status
+            @param playlists as Playlists
+            @param playlist_id as int
+        """
+        exists = playlists.exists(playlist_id)
+        if exists:
+            item = None
+            for child in self._box.get_children():
+                if child.data == playlist_id:
+                    item = child
+                    break
+            if item is None:
+                # Setup sort on insert
+                self._box.set_sort_func(self.__sort_func)
+                self._add_items([playlist_id])
+            else:
+                name = App().playlists.get_name(playlist_id)
+                item.rename(name)
+        else:
+            for child in self._box.get_children():
+                if child.data == playlist_id:
+                    child.destroy()
+
     def __on_new_button_clicked(self, button):
         """
             Add a new playlist
             @param button as Gtk.Button
         """
         existing_playlists = []
-        for (playlist_id, name) in App().playlists.get():
+        for (playlist_id, name, sortname) in App().playlists.get():
             existing_playlists.append(name)
 
         # Search for an available name
@@ -107,7 +159,3 @@ class PlaylistsManagerView(FlowBoxView):
             count += 1
             name = _("New playlist ") + str(count)
         App().playlists.add(name)
-        if self.__obj is None:
-            App().window.container.reload_view()
-        else:
-            App().window.container.show_playlist_manager(self.__obj)
