@@ -10,7 +10,7 @@
 # You should have received a copy of the GNU General Public License
 # along with this program. If not, see <http://www.gnu.org/licenses/>.
 
-from gi.repository import GLib, Gtk, Gio, GObject, Pango
+from gi.repository import GLib, Gtk, Gdk, Gio, GObject, Pango
 
 from gettext import gettext as _
 from collections import OrderedDict
@@ -75,6 +75,8 @@ class TracksView:
         """
         if self._responsive_widget is None:
             self.connect("size-allocate", self.__on_size_allocate)
+            if self._list_type & RowListType.DND:
+                self.connect("key-press-event", self.__on_key_press_event)
             self._responsive_widget = Gtk.Grid()
             self._responsive_widget.set_column_homogeneous(True)
             self._responsive_widget.set_property("valign", Gtk.Align.START)
@@ -325,6 +327,7 @@ class TracksView:
             row.connect("insert-track", self.__on_insert_track)
             row.connect("insert-album", self.__on_insert_album)
             row.connect("remove-track", self.__on_remove_track)
+            row.connect("do-selection", self.__on_do_selection)
         row.show()
         widget.insert(row, position)
         GLib.idle_add(self.__add_tracks, widgets, disc_number, row)
@@ -410,12 +413,26 @@ class TracksView:
             context_widget.destroy()
         return True
 
+    def __on_key_press_event(self, widget, event):
+        """
+            Handle keyboard events (DEL, ...)
+            @param widget as Gtk.Widget
+            @param event as Gdk.EventKey
+        """
+        if event.keyval == Gdk.KEY_Delete:
+            from lollypop.widgets_row_dnd import DNDRow
+            for child in self.children:
+                if child.get_state_flags() & Gtk.StateFlags.SELECTED:
+                    DNDRow.destroy_track_row(child)
+
     def __on_activated(self, widget, track):
         """
             On track activation, play track
             @param widget as TracksWidget
             @param track as Track
         """
+        for child in self.children:
+            child.set_state_flags(Gtk.StateFlags.NORMAL, True)
         # Add to queue by default
         if App().player.is_locked:
             if track.id in App().player.queue:
@@ -444,8 +461,11 @@ class TracksView:
         if track.album.id == row.track.album.id:
             position = self.children.index(row)
             new_row = TrackRow(track, self._list_type)
+            new_row.connect("destroy", self.__on_row_destroy)
             new_row.connect("insert-track", self.__on_insert_track)
+            new_row.connect("insert-album", self.__on_insert_album)
             new_row.connect("remove-track", self.__on_remove_track)
+            new_row.connect("do-selection", self.__on_do_selection)
             new_row.show()
             if down:
                 position += 1
@@ -537,7 +557,7 @@ class TracksView:
     def __on_insert_album(self, row, new_album_id, track_ids, down):
         """
             Insert a new row at position
-            @param row as AlbumRow
+            @param row as Row
             @param new_album_id as int
             @param track_ids as int
             @param down as bool
@@ -554,6 +574,28 @@ class TracksView:
         self.emit("insert-album-after", new_album, split_album)
         self.__destroy_split(row, down)
 
+    def __on_do_selection(self, row):
+        """
+            Select rows from start (or any selected row) to track
+            @param row as Row
+        """
+        children = self.children
+        selected = None
+        end = children.index(row) + 1
+        for child in children:
+            if child == row:
+                break
+            if child.get_state_flags() & Gtk.StateFlags.SELECTED:
+                selected = child
+        if selected is None:
+            start = 0
+        else:
+            start = children.index(selected)
+        for child in children[start:end]:
+            child.set_state_flags(Gtk.StateFlags.SELECTED, True)
+        for child in children[end:]:
+            child.set_state_flags(Gtk.StateFlags.NORMAL, True)
+
     def __on_size_allocate(self, widget, allocation):
         """
             Change box max/min children
