@@ -130,9 +130,9 @@ class RadioWidget(Gtk.FlowBoxChild, OverlayHelper):
         selected = App().player.current_track.id == Type.RADIOS and\
             name == App().player.current_track.radio_name
         if selected:
-            self._artwork.set_state(Gtk.StateType.SELECTED)
+            self._artwork.set_state_flags(Gtk.StateFlags.SELECTED, True)
         else:
-            self._artwork.set_state(Gtk.StateType.NORMAL)
+            self._artwork.set_state_flags(Gtk.StateFlags.NORMAL, True)
 
     @property
     def is_populated(self):
@@ -173,7 +173,7 @@ class RadioWidget(Gtk.FlowBoxChild, OverlayHelper):
         """
         if self._lock_overlay or\
                 self._show_overlay == show_overlay or\
-                App().player.is_locked:
+                (App().player.is_locked and show_overlay):
             return
         OverlayHelper._show_overlay_func(self, show_overlay)
         if show_overlay:
