@@ -13,7 +13,6 @@
 from gi.repository import Gtk, Pango, GLib, Gst, Gdk
 
 from lollypop.define import App, RowListType
-from lollypop.pop_menu import TrackMenuPopover, TrackMenu
 from lollypop.widgets_indicator import IndicatorWidget
 from lollypop.widgets_context import ContextWidget
 from lollypop.utils import seconds_to_string
@@ -144,6 +143,14 @@ class Row(Gtk.ListBoxRow):
             self._num_label.get_style_context().remove_class("queued")
             self._num_label.set_text("")
 
+    @property
+    def row_widget(self):
+        """
+            Get row main widget
+            @return Gtk.Widget
+        """
+        return self._row_widget
+
     @property
     def track(self):
         """
@@ -159,6 +166,7 @@ class Row(Gtk.ListBoxRow):
         """
             Return TrackMenu
         """
+        from lollypop.pop_menu import TrackMenu
         return TrackMenu(self._track)
 
     def _on_destroy(self, widget):
@@ -183,20 +191,36 @@ class Row(Gtk.ListBoxRow):
         self.set_indicator(True, False)
         self.__preview_timeout_id = None
 
-    def __popup_menu(self, widget, xcoordinate=None, ycoordinate=None):
+    def __popup_menu(self, eventbox, xcoordinate=None, ycoordinate=None):
         """
             Popup menu for track
-            @param widget as Gtk.Button
+            @param eventbox as Gtk.EventBox
             @param xcoordinate as int (or None)
             @param ycoordinate as int (or None)
         """
-        popover = TrackMenuPopover(self._track, self._get_menu())
+        from lollypop.pop_menu import TrackMenuPopover, RemoveMenuPopover
+        if self.get_state_flags() & Gtk.StateFlags.SELECTED:
+            # Get all selected rows
+            rows = [self]
+            r = self.previous_row
+            while r is not None:
+                if r.get_state_flags() & Gtk.StateFlags.SELECTED:
+                    rows.append(r)
+                r = r.previous_row
+            r = self.next_row
+            while r is not None:
+                if r.get_state_flags() & Gtk.StateFlags.SELECTED:
+                    rows.append(r)
+                r = r.next_row
+            popover = RemoveMenuPopover(rows)
+        else:
+            popover = TrackMenuPopover(self._track, self._get_menu())
         if xcoordinate is not None and ycoordinate is not None:
-            rect = widget.get_allocation()
+            rect = eventbox.get_allocation()
             rect.x = xcoordinate
             rect.y = ycoordinate
             rect.width = rect.height = 1
-        popover.set_relative_to(widget)
+        popover.set_relative_to(eventbox)
         popover.set_pointing_to(rect)
         popover.connect("closed", self.__on_closed)
         self.get_style_context().add_class("track-menu-selected")
@@ -230,7 +254,7 @@ class Row(Gtk.ListBoxRow):
             self.__menu_button.connect(
                 "button-release-event",
                 self.__on_indicator_button_release_event)
-            self._indicator.button()
+        self._indicator.button()
 
     def __on_leave_notify_event(self, widget, event):
         """
@@ -271,7 +295,17 @@ class Row(Gtk.ListBoxRow):
             @param widget as Gtk.Widget
             @param event as Gdk.EventButton
         """
-        if event.button == 3:
+        if event.state & Gdk.ModifierType.CONTROL_MASK and\
+                self._list_type & RowListType.DND:
+            if self.get_state_flags() & Gtk.StateFlags.SELECTED:
+                self.set_state_flags(Gtk.StateFlags.NORMAL, True)
+            else:
+                self.set_state_flags(Gtk.StateFlags.SELECTED, True)
+                self.grab_focus()
+        elif event.state & Gdk.ModifierType.SHIFT_MASK and\
+                self._list_type & RowListType.DND:
+            self.emit("do-selection")
+        elif event.button == 3:
             window = widget.get_window()
             if window == event.window:
                 self.__popup_menu(widget, event.x, event.y)
@@ -284,7 +318,7 @@ class Row(Gtk.ListBoxRow):
                 App().player.remove_from_queue(self._track.id)
             else:
                 App().player.append_to_queue(self._track.id)
-        elif event.state & Gdk.ModifierType.SHIFT_MASK:
+        elif event.state & Gdk.ModifierType.MOD1_MASK:
             App().player.clear_albums()
             App().player.reset_history()
             App().player.load(self._track)
